@@ -8,12 +8,18 @@ from asyncio import BaseProtocol, BaseTransport, DatagramTransport
 from asyncio.events import AbstractEventLoop
 from datetime import datetime
 from ipaddress import IPv4Address, IPv6Address, ip_address
-from typing import Awaitable, Callable, MutableMapping, Optional, Tuple, cast
+from typing import Awaitable, Callable, Optional, Tuple, cast
 from urllib.parse import urlsplit, urlunsplit
 
 from aiohttp.http_parser import HeadersParser
 
-from async_upnp_client.const import AddressTupleV6Type, AddressTupleVXType, IPvXAddress
+from async_upnp_client.const import (
+    AddressTupleV6Type,
+    AddressTupleVXType,
+    IPvXAddress,
+    SsdpHeaders,
+    UniqueDeviceName,
+)
 from async_upnp_client.utils import CaseInsensitiveDict
 
 SSDP_PORT = 1900
@@ -25,11 +31,7 @@ SSDP_TARGET = SSDP_TARGET_V4
 SSDP_ST_ALL = "ssdp:all"
 SSDP_ST_ROOTDEVICE = "upnp:rootdevice"
 SSDP_MX = 4
-
-SSDP_ALIVE = "ssdp:alive"
-SSDP_UPDATE = "ssdp:update"
-SSDP_BYEBYE = "ssdp:byebye"
-
+SSDP_DISCOVER = '"ssdp:discover"'
 
 _LOGGER = logging.getLogger(__name__)
 _LOGGER_TRAFFIC_SSDP = logging.getLogger("async_upnp_client.traffic.ssdp")
@@ -106,6 +108,15 @@ def is_valid_ssdp_packet(data: bytes) -> bool:
     )
 
 
+def udn_from_headers(headers: SsdpHeaders) -> Optional[UniqueDeviceName]:
+    """Get UDN from USN in headers."""
+    if "usn" in headers and "uuid:" in headers["usn"]:
+        parts = str(headers["usn"]).split("::")
+        return parts[0]
+
+    return None
+
+
 def decode_ssdp_packet(
     data: bytes, addr: AddressTupleVXType
 ) -> Tuple[str, CaseInsensitiveDict]:
@@ -119,7 +130,7 @@ def decode_ssdp_packet(
         lines.append(b"")
 
     parsed_headers, _ = HeadersParser().parse_headers(lines)
-    headers = CaseInsensitiveDict(**dict(parsed_headers))
+    headers = CaseInsensitiveDict(parsed_headers)
 
     # adjust some headers
     if "location" in headers:
@@ -131,9 +142,9 @@ def decode_ssdp_packet(
     headers["_host"] = get_host_string(addr)
     headers["_port"] = addr[1]
 
-    if "usn" in headers and "uuid:" in headers["usn"]:
-        parts = str(headers["usn"]).split("::")
-        headers["_udn"] = parts[0]
+    udn = udn_from_headers(headers)
+    if udn:
+        headers["_udn"] = udn
 
     return request_line, headers
 
@@ -145,7 +156,7 @@ class SsdpProtocol(BaseProtocol):
         self,
         loop: AbstractEventLoop,
         on_connect: Optional[Callable[[DatagramTransport], Awaitable]] = None,
-        on_data: Optional[Callable[[str, MutableMapping[str, str]], Awaitable]] = None,
+        on_data: Optional[Callable[[str, SsdpHeaders], Awaitable]] = None,
     ) -> None:
         """Initialize."""
         self.loop = loop
@@ -180,7 +191,14 @@ class SsdpProtocol(BaseProtocol):
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
         """Handle connection lost."""
-        _LOGGER.debug("Lost connection, transport: %s", self.transport)
+        _LOGGER.debug("Lost connection, error: %s, transport: %s", exc, self.transport)
+
+    def send_ssdp_packet(self, packet: bytes, target: AddressTupleVXType) -> None:
+        """Send a SSDP packet."""
+        _LOGGER.debug("Sending M-SEARCH packet, transport: %s", self.transport)
+        _LOGGER_TRAFFIC_SSDP.debug("Sending M-SEARCH packet: %s", packet)
+        assert self.transport is not None
+        self.transport.sendto(packet, target)
 
 
 def get_source_ip_from_target_ip(target_ip: IPvXAddress) -> IPvXAddress:
