@@ -4,73 +4,92 @@
 import asyncio
 import re
 import socket
+from collections import defaultdict
 from collections.abc import Mapping as abcMapping
-from collections.abc import MutableMapping
+from collections.abc import MutableMapping as abcMutableMapping
 from datetime import datetime, timedelta, timezone
 from socket import AddressFamily  # pylint: disable=no-name-in-module
-from typing import Any, Callable, Dict, Generator, Mapping, Optional, Tuple
+from typing import Any, Callable, Dict, Generator, Optional, Tuple
 from urllib.parse import urljoin, urlsplit
 
+import defusedxml.ElementTree as DET
 from voluptuous import Invalid
 
 EXTERNAL_IP = "1.1.1.1"
 EXTERNAL_PORT = 80
 
 
-def _ci_key(key: str) -> str:
-    """Get storable key from key."""
-    return key.lower()
-
-
-class CaseInsensitiveDict(MutableMapping):
+class CaseInsensitiveDict(abcMutableMapping):
     """Case insensitive dict."""
 
-    def __init__(self, **kwargs: Any) -> None:
+    def __init__(self, data: Optional[abcMapping] = None, **kwargs: Any) -> None:
         """Initialize."""
-        self._data: Dict[str, Any] = dict()
-        for key, value in kwargs.items():
-            self[key] = value
+        self._data: Dict[str, Any] = {**(data or {}), **kwargs}
+        self._case_map: Dict[str, Any] = {k.lower(): k for k in self._data}
+
+    def as_dict(self) -> Dict[str, Any]:
+        """Return the underlying dict without iterating."""
+        return self._data
+
+    def as_lower_dict(self) -> Dict[str, Any]:
+        """Return the underlying dict in lowercase."""
+        return {k.lower(): v for k, v in self._data.items()}
+
+    def replace(self, new_data: abcMapping) -> None:
+        """Replace the underlying dict."""
+        self._data = {**new_data}
+        self._case_map = {k.lower(): k for k in new_data}
 
     def __setitem__(self, key: str, value: Any) -> None:
         """Set item."""
-        key_ci = _ci_key(key)
-        self._data[key_ci] = (key, value)
+        lower_key = key.lower()
+        if self._case_map.get(lower_key, key) != key:
+            # Case changed
+            del self._data[self._case_map[lower_key]]
+        self._data[key] = value
+        self._case_map[lower_key] = key
 
     def __getitem__(self, key: str) -> Any:
         """Get item."""
-        ci_key = _ci_key(key)
-        return self._data[ci_key][1]
+        return self._data[self._case_map[key.lower()]]
 
     def __delitem__(self, key: str) -> None:
         """Del item."""
-        ci_key = _ci_key(key)
-        del self._data[ci_key]
+        lower_key = key.lower()
+        del self._data[self._case_map[lower_key]]
+        del self._case_map[lower_key]
 
     def __len__(self) -> int:
         """Get length."""
         return len(self._data)
 
-    def __iter__(self) -> Generator[Any, None, None]:
+    def __iter__(self) -> Generator[str, None, None]:
         """Get iterator."""
-        return (key for key, value in self._data.values())
+        return (key for key in self._data.keys())
 
     def __repr__(self) -> str:
         """Repr."""
-        return str(dict(self.items()))
+        return repr(self._data)
+
+    def __str__(self) -> str:
+        """Str."""
+        return str(self._data)
 
     def __eq__(self, other: Any) -> bool:
         """Compare for equality."""
-        if not isinstance(other, abcMapping) and not isinstance(other, dict):
-            return NotImplemented
+        if isinstance(other, CaseInsensitiveDict):
+            return self.as_lower_dict() == other.as_lower_dict()
 
-        dict_a = {_ci_key(key): value for key, value in self.items()}
-        dict_b = {_ci_key(key): value for key, value in other.items()}
-        return dict_a == dict_b
+        if isinstance(other, abcMapping):
+            return self.as_lower_dict() == {
+                key.lower(): value for key, value in other.items()
+            }
+
+        return NotImplemented
 
     def __hash__(self) -> int:
         """Get hash."""
-        ci_dict = {_ci_key(key): value for key, value in self.items()}
-        return hash(tuple(sorted(ci_dict.items())))
+        return hash(tuple(sorted(self._data.items())))
 
 
 def time_to_str(time: timedelta) -> str:
@@ -131,7 +150,7 @@ def parse_date_time(value: str) -> Any:
     utc = timezone(timedelta(hours=0))
     if value[-6] in ["+", "-"] and value[-3] == ":":
         value = value[:-3] + value[-2:]
-    matchers: Mapping[str, Callable] = {
+    matchers: Dict[str, Callable] = {
         # date
         r"\d{4}-\d{2}-\d{2}$": lambda s: datetime.strptime(value, "%Y-%m-%d").date(),
         r"\d{2}:\d{2}:\d{2}$": lambda s: datetime.strptime(value, "%H:%M:%S").time(),
@@ -217,3 +236,38 @@ async def async_get_local_ip(
         return sock.family, sockname[0]
     finally:
         transport.close()
+
+
+# Adapted from http://stackoverflow.com/a/10077069
+# to follow the XML to JSON spec
+# https://www.xml.com/pub/a/2006/05/31/converting-between-xml-and-json.html
+def etree_to_dict(tree: DET) -> Dict[str, Optional[Dict[str, Any]]]:
+    """Convert an ETree object to a dict."""
+    # strip namespace
+    tag_name = tree.tag[tree.tag.find("}") + 1 :]
+
+    tree_dict: Dict[str, Optional[Dict[str, Any]]] = {
+        tag_name: {} if tree.attrib else None
+    }
+    children = list(tree)
+    if children:
+        child_dict: Dict[str, list] = defaultdict(list)
+        for child in map(etree_to_dict, children):
+            for k, val in child.items():
+                child_dict[k].append(val)
+        tree_dict = {
+            tag_name: {k: v[0] if len(v) == 1 else v for k, v in child_dict.items()}
+        }
+    dict_meta = tree_dict[tag_name]
+    if tree.attrib:
+        assert dict_meta is not None
+        dict_meta.update(("@" + k, v) for k, v in tree.attrib.items())
+    if tree.text:
+        text = tree.text.strip()
+        if children or tree.attrib:
+            if text:
+                assert dict_meta is not None
+                dict_meta["#text"] = text
+        else:
+            tree_dict[tag_name] = text
+    return tree_dict
